import click
from app import add
from app import check
from app import remove
from app import renew
from app import update

@click.group(help='CLI tool to help you stay in touch with your friends')
@click.version_option(package_name='friendly-reminder')
def cli():
    pass


cli.add_command(add.add)
cli.add_command(check.check)
cli.add_command(remove.remove)
cli.add_command(renew.renew)
cli.add_command(update.update)


if __name__ == '__main__':
    cli()



import click
from app import data
from datetime import date,datetime


@click.command(short_help='Check reminders for today')
@click.option('--all', help='Include friends you contacted recently', is_flag=True)
def check(all):
    today = date.today()
    
    def days_remaining(friend):
        last = datetime.strptime(friend.last_contact, '%Y-%m-%d').date()
        delta = today - last
        return friend.reminder_days - delta.days


    d = data.load()
    friends = {f.name: days_remaining(f)  for f in d.friends}
    if not all:
        new_friends = {}
        for f in friends:
            remaining_days = friends[f]
            if remaining_days <= 0:
                new_friends[f] = friends[f]
        friends = new_friends
    
    def get_order(k):
        return friends[k]

    lst = sorted([f for f in friends], key=get_order)
    if lst:
        print('Think about contacting:')
        for l in lst:
            print(f'{l} ({friends[l]} days)')
    else:
        print('Nothing for today.')



import yaml
import os
from app import constants


class config:

    def __init__(self):
        # Default number of days after which reminders will be shown.
        self.default_days = 7

    def yaml(self):
        return yaml.dump(self.__dict__)

    def load(self, data):
        values = yaml.safe_load(data)
        self.default_days = values['default_days']


def load_config():

    def read_yaml(file):
        with open(file, 'r') as f:
            cfg = config()
            cfg.load(f.read())
            return cfg

    def write_yaml(file, cfg):
        with open(file, 'w') as f:
            f.write(cfg.yaml())

    if os.path.isfile(constants.CONFIG_FILE_PATH):
        return read_yaml(constants.CONFIG_FILE_PATH)
    else:
        cfg = config()
        if not os.path.exists(constants.CONFIG_DIR):
            os.mkdir(constants.CONFIG_DIR)
        write_yaml(constants.CONFIG_FILE_PATH, cfg)
        return cfg



import click
from app import data
from datetime import datetime


@click.command(short_help='Change settings for a contact')
@click.option('--name', help='The name of the person you want to update', required=True)
@click.option('--new-name', help='A new name for the friend')
@click.option('--days', help='How many days between reminders', type=int)
@click.option('--last-contact', help='Date of your last contact in the format yyyy-mm-dd')
def update(name, new_name, days, last_contact):
    d = data.load()
    friend = next(filter(lambda f: f.name == name, d.friends), None)
    if not friend:
        raise click.UsageError(f'{name} is not in the reminder list.')
    if any(filter(lambda f: f.name == new_name, d.friends)):
        raise click.UsageError(f'{new_name} is already in the reminder list.')
    if new_name:
        friend.name = new_name
    if days:
        friend.reminder_days = days
    if last_contact:
        try:
            date = datetime.strptime(last_contact, '%Y-%m-%d')
            friend.last_contact = last_contact
        except:
            raise click.UsageError(f'{last_contact} is not a valid date as yyyy-mm-dd.')
    d.save()
    print(f'Updated {name} succesfully.')



import click
from app import data
from datetime import date


@click.command(short_help='Remove a contact from the reminder list')
@click.option('--name', help='The name of the person to remove', required=True)
def remove(name):
    d = data.load()
    friend = next(filter(lambda f: f.name == name, d.friends), None)
    if friend:
        d.friends.remove(friend)
    d.save()
    print(f'Removed {name} succesfully.')



import click
from app import data
from app import config
from datetime import date


@click.command(short_help='Add a new contact')
@click.option('--name', help='The name of the person you want to stay in touch with', required=True)
@click.option('--days', help='How many days between reminders', type=int)
def add(name, days):
    d = data.load()
    if any(filter(lambda f: f.name == name, d.friends)):
        raise click.UsageError(f'{name} is already in the reminder list.')
    c = config.load_config()
    today = date.today()
    day = today.strftime('%Y-%m-%d')
    if not days:
        days = c.default_days
    d.friends.append(data.friend(name, days, day))
    d.save()
    print(f'Added {name} succesfully.')



import click
from app import data
from datetime import date


@click.command(short_help='Restart the reminder period for a contact')
@click.option('--name', help='The name of the person you contacted', required=True)
def renew(name):
    d = data.load()
    friend = next(filter(lambda f: f.name == name, d.friends), None)
    if not friend:
        raise click.UsageError(f'{name} is not in the reminder list.')
    today = date.today()
    day = today.strftime('%Y-%m-%d')
    friend.last_contact = day
    d.save()
    print(f'Updated {name} succesfully.')



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

setup(
    name = 'friendly-reminder',
    version = '1.0.0',
    author = 'Robert Lützner',
    author_email = 'robert.luetzner@pm.me',
    license = 'GPL-3.0',
    description = 'A CLI tool to stay on touch with your friends',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = 'https://github.com/rluetzner/friendly-reminder',
    py_modules = ['main', 'app'],
    packages = find_packages(),
    install_requires = [requirements],
    python_requires='>=3.6',
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Operating System :: OS Independent",
    ],
    entry_points = '''
        [console_scripts]
        friendly-reminder=main:cli
    '''
)



import os
import jsonpickle
from app import constants


class friend:

    def __init__(self, name, reminder_days, last_contact):
        self.name = name
        self.reminder_days = reminder_days
        self.last_contact = last_contact

class data:

    def __init__(self):
        self.friends = []

    def save(self):
        if not os.path.exists(constants.CONFIG_DIR):
            os.mkdir(constants.CONFIG_DIR)
        with open(constants.DATA_FILE_PATH, 'w') as f:
            f.write(jsonpickle.encode(self, indent=4, unpicklable=False))


def load():

    if os.path.isfile(constants.DATA_FILE_PATH):
        with open(constants.DATA_FILE_PATH, 'r') as f:
            dec = jsonpickle.decode(f.read())
            d = data()
            for f in dec['friends']:
                d.friends.append(friend(f['name'], f['reminder_days'], f['last_contact']))
            return d
    else:
        return data()



import os
from pathlib import Path


HOME = Path.home()
CONFIG_DIR = os.path.join(HOME, '.config', 'friendly-reminder')
DATA_FILE_PATH = os.path.join(CONFIG_DIR, 'data.json')
CONFIG_FILE_PATH = os.path.join(CONFIG_DIR, 'friendly-reminder.yml')
